// 3D starfield
//
// Programmed in Turbo C Version 3.0
// Can be compiled under the small model
//
// Mark Bryant
// Compuserve 100044.256
//****************************************************************************
//
// The formula used is: X=screen distance*Xpos/Zpos
//                      Y=screen distance*Ypos/Zpos
//
// To speed up the calculations the screen distance (100) is pre-calculated
// when the random positions are created.
// The only movement is in the Z plane. Zpos starts at 512 and is decreased
// to 1. This value is used as 512/255(number of colours)=2 so dividing
// Zpos by 2 gives a the colour of the star.
//


#include "stdlib.h"

void video(int mode);
void wait_retrace(void);
void newstar(void);

// Set up global variables

int spx[500], spy[500], spz[500];      // stars x,y & z positions
int srp[1000];                         // pre-calculated random positions
int sl;
int nsp=0;

main()
{

// Set up local variables

int a,b,x,y;
char far *pixel;  		        // far pointer to video memory
char key;
int ts;
int px[500],py[500];                    // stars screen positions
int r1,r2,r3;
int speed=2;
int kd=0;

int palr[255];                          // pallete RGB arrays
int palg[255];
int palb[255];


     video(0x03);                       // clear the screen
     printf ("Hit ESCAPE to exit \n");
     printf ("F11 and F12 change the speed \n \n");
     printf("How many stars (max 500) ?");
     scanf("%i",&ts);
     ts=ts-1;

     // Create the stars random positions
     randomize();
     for (x=0; x<=ts; x++)
     {
     r1=random(640)-320;
     r2=random(400)-200;
     r3=random(510);

     spx[x]=r1*100;
     spy[x]=r2*100;
     spz[x]=r3;
     }

     for (x=0; x<1000 ; x=x+2)
     {
     r1=random(640)-320;
     r2=random(400)-200;
     srp[x]=r1*100;
     srp[x+1]=r2*100;
     }


     video(0x13);     // Change the video mode to 320*200  256 colours

     // define the pallete
     // colour 1 is white gradually fading to black at colour 255
     a=63;
     b=4;
     for (x=1;x<=255;x++)
     {
     b=b-1;
     if (b==0) { a=a-1; b=4; }
     palr[x]=a;
     palg[x]=a;
     palb[x]=a;

     outportb(0x3c8,x);
     outportb(0x3c9,a);
     outportb(0x3c9,a);
     outportb(0x3c9,a);
     }


// Main Loop

 do
 {
     key=inport(0x60);        // Read the keyboard hardware port

     // check for F11 & F12
     kd=kd+1;
     if (kd==10)
     {
     if (key==87) speed=speed-1;
     if (key==88) speed=speed+1;
     if (speed==0) speed=1;
     kd=0;
     }


      wait_retrace();                  // Wait for the vertical retrace

      for (sl=0 ; sl<=ts ; sl++)
      {

       pixel = (char far *)0xA0000000 + (py[sl]*320) + px[sl];
       pixel[0] = 0;                    // clear the old position

       spz[sl]=spz[sl]-speed;           // decrease z
       if (spz[sl]<=0) newstar();

       px[sl]=(spx[sl]/spz[sl])+160;    // calculate the screen position
       py[sl]=(spy[sl]/spz[sl])+100;    // with an offset of the screen centre

       if (px[sl]<0) newstar();
       if (px[sl]>319) newstar();
       if (py[sl]<0) newstar();
       if (py[sl]>199) newstar();

       pixel = (char far *)0xA0000000 + (py[sl]*320) + px[sl];
       pixel[0] = spz[sl]/2;          // plot the new star with a colour
				      // depending on its distance
       }
    }
    while (key!=1);


   video(0x03);                    // Text mode
   return(0);

}


// Functions


void video(int mode)
{
asm {
mov ax,mode
int 0x010 }
}

void wait_retrace()
{
asm {

mov dx,0x3da
}
bra1:
asm {
	    in  al,dx
	    and al,8
	    jnz bra1
}
bra2:
asm {
	    in  al,dx
	    and al,8
	    jz bra2
}
}

void newstar()
{
spx[sl]=srp[nsp];
spy[sl]=srp[nsp+1];
spz[sl]=510;
nsp=nsp+1;
if (nsp>1000) nsp=0;
}